import { subspaced } from 'react-redux-subspace';
import { defaultStateHandler } from '@redux-dynostore/core';

/**
 * Copyright 2018, IOOF Holdings Limited.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree.
 */

var subspacedEnhancer = function subspacedEnhancer(_temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      stateHandler = _ref.stateHandler,
      _ref$mapExtraState = _ref.mapExtraState,
      mapExtraState = _ref$mapExtraState === void 0 ? function () {
    return null;
  } : _ref$mapExtraState;

  return function (identifier) {
    return function (store) {
      var defaultOptions = store.dynostoreOptions || {};

      var _ref2 = stateHandler || defaultOptions.stateHandler || defaultStateHandler,
          getValue = _ref2.getValue,
          canMerge = _ref2.canMerge,
          merge = _ref2.merge;

      var mapState = function mapState(state, rootState) {
        var componentState = getValue(state, identifier);

        if (!canMerge(componentState)) {
          return componentState;
        }

        var extraState = mapExtraState(state, rootState);

        if (!canMerge(extraState)) {
          if (process.env.NODE_ENV !== 'production' && extraState !== null) {
            throw new TypeError("extra state must be a mergable with the component state but received " + extraState);
          }

          return componentState;
        }

        return merge(extraState, componentState);
      };

      var subspaceEnhancer = subspaced(mapState, identifier);
      return function (Component) {
        return subspaceEnhancer(Component);
      };
    };
  };
};

export default subspacedEnhancer;
